package token

import (
	"bytes"
)

// List is a slice of tokens.
type List []Token

// IndexKind returns the position of a token kind within a token list.
func (list List) IndexKind(kind Kind) int {
	for i, token := range list {
		if token.Kind == kind {
			return i
		}
	}

	return -1
}

// LastIndexKind returns the position of the last token kind within a token list.
func (list List) LastIndexKind(kind Kind) int {
	for i := len(list) - 1; i >= 0; i-- {
		if list[i].Kind == kind {
			return i
		}
	}

	return -1
}

// String implements string serialization.
func (list List) String() string {
	builder := bytes.Buffer{}
	var last Token

	for _, t := range list {
		if last.Kind == Keyword || last.Kind == Separator || last.Kind == Operator || t.Kind == Operator {
			builder.WriteByte(' ')
		}

		builder.Write(t.Bytes)
		last = t
	}

	return builder.String()
}
